'''
Created on Dec 1, 2023

@author: Claudio Scafuri
Classes for extracting and formatting some data from lattice files, maninly Elttra 2.0.
Intended usage: auxiliary for SVG synoprtic renderer, generation of tango device names
'''


# TODO: get TYPE of magnets etc... which must be got from another source


import at # based on PyAt package


class ElemTag(object):
    '''
    Class for holding and formatting lattice element data in the layout
    
    '''
    
    def __init__(self,i,ename,length,x,y,angle,spos,iscorr,isskew,device,strength):
        self.ind = i
        self.name = ename
        self.length = length
        self.x = x
        self.y = y
        self.angle = angle
        self.spos = spos
        self.iscorr = iscorr
        self.isskew = isskew
        self.device = device
        self.strength = strength
    def field_names(self,sepa='\t'):
        return('i'+sepa+'name'+sepa+'length'+sepa+'x'+sepa+'y'+sepa+'angle'+sepa+'spos'+sepa+'iscorr'+sepa+'isskew'+sepa+'device'+sepa+'strength')
    def printout(self,sepa='\t'):
        print(self.ind,sepa,self.name,sepa,self.length,sepa, self.x,sepa,self.y,sepa,self.angle,sepa,self.spos,sepa,self.iscorr,sepa,self.isskew,sepa,self.device,sepa,self.strength)        
       
class LatticeInfo(object):
    '''
    Class for extracting relevant elements data from the at/pyat lattice file
    '''
    def __init__(self,ringname,ene):
        self.ring = at.load_lattice(ringname,energy=ene)
        self.geo, radius=self.ring.get_geometry((0,0,0),True)
        self._addlists()
    def _lister(self,selector):
        elist=[]
        indexes = self.ring.uint32_refpts(selector)
        for i in indexes:
            elem = self.ring[i]
            ename = elem.FamName
            length = elem.Length
            x=self.geo[i].x
            y=self.geo[i].y
            angle=self.geo[i].angle
            if hasattr(elem,'corID'):
                iscorr = True
            else:
                iscorr = False
            if hasattr(elem,'SkewID'):
                isskew = True
            else:
                isskew = False
            if hasattr(elem,'Device'):
                device = elem.Device
            else:
                device =''
            strength = ''
            try:
                if elem.K != 0.0:
                    strength =  str(elem.K)
                if elem.H != 0.0:
                    strength = str(elem.H)
            except:
                strength =''
            spos = self.ring.get_s_pos(i)[0]
            eletag = ElemTag(i,ename,length,x,y,angle,spos,iscorr,isskew,device,strength)
            elist.append(eletag)
        return elist
    def _addlists(self):
        self.quads=self._lister(at.Quadrupole)
        self.antibends=self._lister("QAB*")
        self.sexts=self._lister(at.Sextupole)
        self.octs=self._lister(at.Octupole)
        self.b64s=self._lister("b1") ## what about b80??
        self.bpms=self._lister("bpm*")
        self.chvs=self._lister("chv*")
        self.rfs=self._lister("RF*")
    def printall(self,sepa='\t'):
        for el in self.quads:
            el.printout()
        for el in self.antibends:
            el.printout()
        for el in self.sexts:
            el.printout()
        for el in self.octs:
            el.printout()
        for el in self.b64s:
            el.printout()
        for el in self.bpms:
            el.printout()
        for el in self.chvs:
            el.printout()
        for el in self.rfs:
            el.printout()
    def field_names(self,sepa='\t'):
        return self.quads[0].field_names(sepa)
